package hiit.parser;

import hiit.parser.rule.AliveAccelerationRule;
import hiit.parser.rule.BetelgeuseParserRule;
import hiit.parser.rule.BluetoothProximityRule;
import hiit.transceiver.Transceiver;
import hiit.util.DbConnectivity;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import java.util.Arrays;
import java.util.Vector;
import java.util.Hashtable;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class BetelgeuseProtocolSocket implements ContextParser {
	// FIXME: Test running with BeTelGeuse
	private String bta;
	private int betSession;
	private long dbSession;
	private int compression;
	private String userName;
	
	private boolean isRunning = true;
	private boolean connectionLost = false;
	private boolean isDead = false;
	private int sensorID = 0;
	
	//If connection is lost the object goes dead after 5mins. It needs to be cleaned up by calling the close() method.
	private long timeBeforeDead = 300000;
	private Connection conn;
	private Statement statement;
	
	private Transceiver trans;
	
	private Vector<BetelgeuseParserRule> rules;
	private Hashtable<Long, Vector<BetelgeuseParserRule>> readerRules = new Hashtable<Long, Vector<BetelgeuseParserRule>>();
	private Hashtable<Long, Vector<BetelgeuseParserRule>> paramRules = new Hashtable<Long, Vector<BetelgeuseParserRule>>();
	
	
	
	/**
	 * Creates a new instance to handle a connection to the socket.
	 * @param bta The bt address of the connected BTG to handle. Might change to a generic identifier in the future.
	 * @param betSession The session id assigned to the connected BTG. Pooled thread session id, unique among connected
	 * BTGs.
	 * @param dbSession The session id on the database id. Unique among all connections that ever connected to the db. 
	 * @param compression sets the compression algorithm to use. Not in use at the moment.
	 * @param s The socket that was accepted and opened for the incoming connection.
	 * @param out The PrintWriter that uses the OutputStream of the socket s.
	 * @param in The InputStream from socket s. 
	 */
	
	public BetelgeuseProtocolSocket(String userName, String bta, int betSession, long dbSession, Transceiver trans){
		this.userName = userName;
		this.bta = bta;
		this.betSession = betSession;
		this.dbSession = dbSession;
		this.trans = trans;
		this.rules = new Vector<BetelgeuseParserRule>();
		this.rules.add(new AliveAccelerationRule());
		this.rules.add(new BluetoothProximityRule());
		
	}
	
	
	/**
	 * Starts the handler thread. Run only exits after a quit message is received 
	 * from BTG or in the case of a database error.
	 */
	
	public void run() {
		// Database initialization.
		this.conn = DbConnectivity.getConnection();
		statement = null;
		try{
			statement = conn.createStatement();
		} catch (SQLException se){
			System.out.println("Unable to create statement. "+ se);
		}
		
		// Maps local sensorIds to database sensor ids.
		Hashtable<Integer, Long> sensorId_dbSensor = new Hashtable<Integer, Long>();
		// maps a sensor to its parameter mapping.
		Hashtable<Long, Hashtable<Integer, Long>> paramMappings = new Hashtable<Long, Hashtable<Integer, Long>>();
		
		Hashtable<Long, Integer> typeMapping = new Hashtable<Long, Integer>();
		Hashtable<Long, String> dbId_paramNameMapping = new Hashtable<Long, String>();
		
		while(isRunning){
			
			//Ok... The connection might be lost for some reason. However, BeTelGeuse might
			//reconnect to the server again...
			if (connectionLost){
				//Try to close the connection...
				this.trans.close();
				
				//Wait in case the object gets a new socket
				synchronized (this){
					try {
						this.wait(timeBeforeDead);
					} catch (InterruptedException e) {
						// Woken up, now we can continue
					}
					
					//"Soft" exit...
					if(!isRunning)
						return;
					
					//So we waited the whole time...
					// ... and were not restored with readFrom() 
					if (connectionLost){
						//The object is dead
						isDead = true;
						// end of run
						return;
					}					
				}
			}
			
			try {				
				int[] head = new int[6]; // 1 Command 3 Length 2 SessionId
				
				int num = this.trans.read(head, 0, 6);
				if (num != 6){ // Remote connection closed or invalid message...
					// ...Hackers HTTP-GET-polling our server... 
					System.err.println("Message header not found! Stopping thread."+ Arrays.toString(head));
					connectionLost = true;
					this.trans.close();
					continue;
				}
				else if (head[0] == -1){
					// End of stream
					System.err.println("End of stream reached! Stopping thread."+ Arrays.toString(head));
					connectionLost = true;
					this.trans.close();
					continue;
				}
				
				int command = head[0];
				int length = head[1]*256*256 + head[2]*256 + head[3] -2; // sessionID already read 
				int btsession = head[4]*256 + head[5];
				if (length < 0){
					//System.out.println("Invalid length.");
					continue;
				}
				
				int offset = 0;

				int[] fish = new int[length];
				num =  this.trans.read(fish, offset, length);

				
				// DEBUG: print all received message headers. Don't print message if it's longer than
				// 512 characters to make reading it easier.
				System.out.print("RECV: " + (char)command + " ");
				System.out.print((length + 2) +" "); // display length given by BTG, not the one used for reading
				System.out.print(btsession + " ");
				if (length > 0){
				    if (length > 512){
					System.out.println(" [message body]");
				    }
				    else{
					System.out.println(Arrays.toString(fish));
				    }
				}
				else 
				    System.out.println(" []");
				// END_DEBUG
				
				if (num != length){ // read more/less than the length field.
					System.err.println("Message body too short: "+ Arrays.toString(fish) + " length < "+ (length));
					continue;
				}
				
				// check the message...
				// default case == invalid message
				
				switch (command){
				case 's': case 'S': // sensor configuration message.
					
					//Start constructing the sensor reply
					//String reply = "I";
					ByteArrayOutputStream reply = new ByteArrayOutputStream();
					if(this.betSession != btsession){ // We are handling a different BTG than the session id given to us.
						// Impossible. Threads mixed up?
						System.err.println("Generic connection error. Please contact the administration...");
					}
					//Read the number of sensors
					int numberOfSensors = fish[offset]*256 + fish[offset + 1];
					offset += 2;
					
					System.out.println("sensors: " + numberOfSensors + " ");
					
					//Loop the sensors
					for (int i = 0; i < numberOfSensors; i++){
						//Sensor ID assigned by BTG
						int btgID = fish[offset] * 256 + fish[offset+1];
						offset +=2;
						//Get the bluetooth address...
						String bta = new String(fish, offset, 12);
						offset += 12;
						System.out.print("bta: " + bta);
						//And the sensor type...
						int type = fish[offset]*256 + fish[offset + 1];
						offset += 2;
						System.out.print(" type: " + type);
						//And the length of the BT name...
						int nameLength = fish[offset];
						offset++;
						System.out.print(" namelt: " + nameLength);
						//And the BT name...
						String btName = new String(fish, offset, nameLength);
						offset += nameLength;
						
						//And number of param IDs...
						int paramCount = fish[offset];
						offset += 1;
						
						//And the parameter field
						btName = btName.replaceAll("'", "''");
						
						// New db:
						/*
						 * sensor is used to identify a single sensor device.
						 * Which means that we must add a line every time we get the sensor config message.
						 * (Otherwise there could not be for example two GPS devices)
						 */
						/* Relevant tables:
						 bluetooth:
						 +--------------+--------------+------+-----+---------+----------------+
						 | Field        | Type         | Null | Key | Default | Extra          |
						 +--------------+--------------+------+-----+---------+----------------+
						 | id           | int(11)      |      | PRI | NULL    | auto_increment |
						 | sensor_id    | int(11)      |      | MUL | 0       |                |
						 | btaddress    | varchar(255) |      |     |         |                |
						 | friendlyname | varchar(255) | YES  |     | NULL    |                |
						 +--------------+--------------+------+-----+---------+----------------+
						 
						 sensor:
						 +-------------+---------------+------+-----+---------+----------------+
						 | Field       | Type          | Null | Key | Default | Extra          |
						 +-------------+---------------+------+-----+---------+----------------+
						 | id          | int(11)       |      | PRI | NULL    | auto_increment |
						 | name        | varchar(255)  |      |     |         |                |
						 | created     | timestamp(14) | YES  |     | NULL    |                |
						 | description | blob          | YES  |     | NULL    |                |
						 +-------------+---------------+------+-----+---------+----------------+
						 */
						
						int success = -1;
						long dbSensor = 0;
						// This is not used at the moment.
						//long btId = 0;
						
						ResultSet rs = null;
						String update = null;
						
						String sensorAdd = "insert into sensor values(0, '"+ type +"', CURRENT_TIMESTAMP, 'A sensor of the type "+ type + "');";
						try{
							success = statement.executeUpdate(sensorAdd, Statement.RETURN_GENERATED_KEYS);
						}catch (SQLException e){
							System.err.println("Unable to add the new sensor to the db: "+ e);
							this.close();
							return;
						}
						if (success != 1){
							System.err.println("Unable to add the new sensor to the db: "+ success);
							this.close();
							return;
						}
						try{
							rs = statement.getGeneratedKeys();
							rs.first();
							dbSensor = rs.getLong(1);
						}catch (SQLException e){
							System.err.println("Unable get added sensor id: "+ e);
							this.close();
							return;
						}
						
						// add the line to the bluetooth table that points to the right sensor line. 
						update = "insert into bluetooth values(0, "+ dbSensor + ", '"+ bta+"', '"+ btName+"', NULL);";
						// NULL, no user reference by default
						success = -1;
						try{
							success = statement.executeUpdate(update, Statement.RETURN_GENERATED_KEYS);
						}catch (SQLException e){
							System.err.println("Unable to add a new bluetooth sensor " + update + ", exception: " + e);
							this.close();
							return;
						}
						if (success != 1){
							System.err.println("Unable to add a new bluetooth sensor. "+ success);
							this.close();
							return;
						}
						// This is not used at the moment
						/*
						try{
							rs = statement.getGeneratedKeys();
							rs.first();
							btId = rs.getLong(1);
						}catch (SQLException e){
							System.err.println("Unable get added bluetooth id: "+ e);
							this.close();
							return;
						}
						*/
						
						update = null;
						int status = -1;
						//Add the BTG id to the reply
						sensorId_dbSensor.put(new Integer(btgID), new Long(dbSensor));
						intToBytes(btgID, 2, reply);
						intToBytes(paramCount,1, reply);
						sensorID++;
						
						//FIXME Does this really work?
						for(BetelgeuseParserRule pr : this.rules){
							int[] temp = pr.getReaderTypes();
							for(int i_ = 0; i_ < temp.length; i_++){
								if(temp[i_] == type){
									Vector tempVec = this.readerRules.get(dbSensor);
									if(tempVec == null){
										tempVec = new Vector<BetelgeuseParserRule>();
										tempVec.add(pr);
										this.readerRules.put(dbSensor, tempVec);
										typeMapping.put(dbSensor, type);
									}else{
										if (!tempVec.contains(pr)){
											tempVec.add(pr);
											typeMapping.put(dbSensor, type);
										}
									}
								}
							}
						}
						
						//Create Hashtable for mapping the BTG parameter IDs with database ones
						Hashtable<Integer, Long> mapping = new Hashtable<Integer, Long>();
						//and put it to the vector... the positions should be exactly the same as the btg sensor id of the sensor.
						paramMappings.put(dbSensor, mapping);
						//btg parameter IDs start at 0...
						int localParamID = 0;
						
						//Get a DB parameter, assing a btg one, add it to the message, map the btg and db ones...
						for(int k = 0; k < paramCount; k++){
							int btgParamId = fish[offset];
							offset++;
							int parLength = fish[offset]*256 + fish[offset+1];
							offset += 2;
							String param = new String(fish, offset, parLength);
							offset += parLength;
							param.replaceAll("'", "''");
							
							long dbId = 0;
							
							String check ="select id from contextatom where datatype='"+param+"';";
							
							try {
								rs = statement.executeQuery(check);
								if(rs.first())
									dbId = rs.getInt("id");
								else {
									// id, name, datatype, created, description
									// we are interested in id and datatype
									update ="insert into contextatom values(0, '"+param +"', '"+param +"', CURRENT_TIMESTAMP, '"+param + " data');";
									status = statement.executeUpdate(update, Statement.RETURN_GENERATED_KEYS);
									if (status == 1){
										rs = statement.getGeneratedKeys();
										if(rs.first())
											dbId = rs.getLong(1);
									}
									else {
										System.err.println("Unable to add or find id for contextatom.");
										this.close();
										return;
									}
								}
							} catch (SQLException e) {
								System.err.println("Unable to query for data types. " + e);
								this.close();
								return;
							}
							
							// DEBUG
							//	System.out.println("Pairing device "+ btName + " with measurement type "+ param);
							// END DEBUG
							
							// Need to add a line to sensorvalue for each contextatom that points to this sensor				
							try{			
								success = statement.executeUpdate("insert into sensorvalue values(0, "+ dbSensor + ", "+ dbId + ");"); 
							} catch (SQLException e){
								System.err.println("Unable to pair sensor with measurement. " + e);
								this.close();
								return;
							}
							if (success != 1){
								System.err.println("Unable to pair sensor with measurement. " + success);
								this.close();
								return;
							}
							
							mapping.put(btgParamId, dbId);
							intToBytes(btgParamId, 1, reply);
							localParamID++;
							
							dbId_paramNameMapping.put(dbId, param);
							
							//FIXME does this really work?
							for(BetelgeuseParserRule pr : this.rules){
								String[] temp = pr.getParamNames();
								for(int i_ = 0; i_ < temp.length; i_++){
									if(temp[i_].equals(param)){
										Vector tempVec = this.paramRules.get(dbId);
										if(tempVec == null){
											tempVec = new Vector<BetelgeuseParserRule>();
											tempVec.add(pr);
											this.paramRules.put(dbId, tempVec);
										}else{
											if (!tempVec.contains(pr)){
												tempVec.add(pr);
											}
										}
									}
								}
							}
						}
					}
					byte[] tempBytes = reply.toByteArray();
					reply = new ByteArrayOutputStream();
					reply.write("I".getBytes());
					intToBytes(tempBytes.length+4,3,reply);
					intToBytes(this.betSession, 2, reply);
					intToBytes(numberOfSensors, 2, reply);
					reply.write(tempBytes);
					
					System.out.print("SEND: ");
					System.out.println("I " + reply.size() + " " + Arrays.toString(reply.toByteArray()));
					
					this.trans.write(reply.toByteArray());
					
					break;
					
				case 'd': case 'D':
					
					/*
					 *    1        3        2               2                  n
					 *  --------------------------------------------------------------------------
					 *	| Cmd | Length |Session ID | Number of blocks |    Data     | ... |     Data     |
					 *	--------------------------------------------------------------------------
					 *   'D'      int       int          int 
					 *  |    handled already       | to be handled.....
					 */
					ResultSet rs = null;
					
					int blocks = fish[offset]*256 + fish[offset +1];
					offset += 2;
					
					for (int j = 0; j < blocks; ++j){
						int nopTotal = 0;
						//Sensor ID
						int sensorid = fish[offset]*256 + fish[offset + 1];
						offset +=2;
						//Timestamp
						int tl = fish[offset];
						offset++;
						long timestamp = -1;
						try{
						    timestamp = Long.parseLong(new String(fish, offset, tl));
						} catch (NumberFormatException n){
						    System.out.println("Could not read timestamp: " + new String(fish, offset, tl) + " from message: " + Arrays.toString(fish));
						}
						offset += tl;
						java.sql.Timestamp time = new java.sql.Timestamp(timestamp);
						
						//Add new contextelement row for the block...
						long elementid = -1;
						// add the received line timestamp.
						String update = "insert into contextelement values(0, CURRENT_TIMESTAMP, 1, '" + time.toString() + "', "+dbSession+");";
						
						int success = -1;
						try {
							success = statement.executeUpdate(update, Statement.RETURN_GENERATED_KEYS);
						} catch (SQLException e) {
							System.err.println("Update failed. "+ update + " exception: "+ e);
							this.close();
							return;
						}
						if (success != 1) {
							System.err.println("Update failed. "+ update + " code: "+ success);
							this.close();
							return;
						}
						try{
							rs = statement.getGeneratedKeys();
							if (!rs.first()){
								System.err.println("Unable to get id for added contextelement. ");
								this.close();
								return;
							}
							elementid = rs.getLong(1);
						}catch (SQLException e){
							System.err.println("Unable to get id for added contextelement. " +e);
							this.close();
							return;
						}
						
						/*
						// We have elementid.
						// sessiondata: add the line:  0, dbSession, elementid
						update = "insert into sessiondata values(0, " + dbSession + ", " + elementid + ");"; 
						try {
							success = statement.executeUpdate(update);
						} catch (SQLException e) {
							System.err.println("Update failed. "+ update + " exception: "+ e);
							this.close();
							return;
						}
						if (success != 1) {
							System.err.println("Update failed. "+ update + " code: "+ success);
							this.close();
							return;
						}
						// session linked to the received line.
						*/
						//Number of parameters
						int nop = fish[offset];
						offset++;
						
						// sensorID -> use it to index the paramMappings vector...
						Hashtable<Integer, Long> map = paramMappings.get(sensorId_dbSensor.get(sensorid));
						
						/*  New db:
						 relevant tables (in order of topological adding):
						 contextelement: add the line: 0, CURRENT_TIMESTAMP, nop
						 save the generated id into elementid
						 sessiondata: add the line:  0, dbSession, elementid
						 contextdata: add a line for each received measurement:
						 elementid, dbid, dbSensor, value
						 bluetooth: add a line for each _new_ discovered device if 
						 bluetooth data is received
						 bluetoothproximity: add a link to contextelement from each bluetooth
						 address that was in the data
						 dbSensor is accessible from:
						 Long dbSensor = (Long)sensorId_dbSensor.get(new Integer(sensorid));
						 */
						
						/*	
						 contextelement: add the line: 0, CURRENT_TIMESTAMP, nop
						 save the generated id into elementid
						 
						 */
						
						long dbSensor = sensorId_dbSensor.get(sensorid).longValue();
						
						//FIXME does this really work?
						//Check if there are any special rules for this sensor...
						Vector<BetelgeuseParserRule> tempVec = this.readerRules.get(dbSensor);
						
						if(tempVec != null){
							int offset2 = offset;
							ByteArrayOutputStream bout = new ByteArrayOutputStream();
							for (int i = 0; i < nop; ++i){
								//bout.write(fish[offset2]);
								offset2++;
								int parLen = fish[offset2]*256 + fish[offset2 + 1];
								//bout.write(fish[offset2]);
								offset2 += 2;
								//bout.write((new String(fish, offset2, parLen)).getBytes());
								offset2 +=parLen;
							}
							int[] tempMessage = new int[offset2 - offset];
							for(int i=0;i<offset2-offset;i++){
								tempMessage[i] = fish[offset+i];
							}
							String tempParams = bout.toString();
							boolean add = true;
							for(int i = 0; i < tempVec.size(); i++){
								if(!tempVec.elementAt(i).addSensorData(typeMapping.get(dbSensor)))
									add = false;
								nopTotal += tempVec.elementAt(i).process(statement, tempMessage, elementid, nop, dbSensor, map, dbId_paramNameMapping, typeMapping.get(dbSensor), timestamp);
							}
							if(!add){
								offset = offset2;
								continue;
							}
						}
						
						// process parameters:
						for (int i = 0; i < nop; ++i){						
							/*  
							 *      1         1       x         1        1        x               1        1        x
							 * ------------------------------------------------------------------------------------------
							 * | Param ID | Length | Value | Param ID | Length | Value | ... | Param ID | Length | Value |
							 * ------------------------------------------------------------------------------------------
							 *     int       int     String     int       int    String    ....
							 */
							
							int paramid = fish[offset];
							offset++;
							long dbid = map.get(paramid);
							
							int paramLength = fish[offset]*256 + fish[offset + 1];
							offset += 2;
							if (paramLength == 0){ // Not adding empty data to the database...
								continue;
							}
							String value = new String(fish, offset, paramLength);
							offset += paramLength;
							
							//FIXME Does this really work?
							//Check if the parameter needs to be processed by another algorithm
							tempVec = this.paramRules.get(dbid);
							if(tempVec != null){
								boolean add = true;
								for(int k = 0; k < tempVec.size(); k++){
									if(!tempVec.elementAt(k).addParameterData(dbId_paramNameMapping.get(dbid)))
										add = false;
									nopTotal += tempVec.elementAt(k).process(statement, value, elementid, dbSensor, dbid, dbId_paramNameMapping.get(dbid), time.getTime());
								}
								if(!add)
									continue;
							}
							
							value = value.replaceAll("'", "''");
							
							//TODO fixthis later
							if(value.equals("\\"))
								continue;
							/* contextdata: add a line for each received measurement:
							 elementid, dbid, dbSensor, value */

							try {
								// Add each data line:
								update = "insert into contextdata values(0," + elementid + ", "+ dbid + ", " + dbSensor + ", '" + value + "');";
								success = statement.executeUpdate(update);
								nopTotal++;
							} catch (SQLException e) {
								System.err.println("Update failed. "+ update + " exception: "+ e);
								this.close();
								return;
							}
							if (success != 1) {
								System.err.println("Update failed. "+ update + " error: "+ success);
								this.close();
								return;
							}
						} // end of process parameters loop
						// then update the true number of measurements received to the received line
						update = "update contextelement set nro_measurements=" + nopTotal + " where id=" + elementid + ";";
						success = -1;
						try {
							success = statement.executeUpdate(update, Statement.RETURN_GENERATED_KEYS);
						} catch (SQLException e) {
							System.err.println("Update failed. "+ update + " exception: "+ e);
							this.close();
							return;
						}
						if (success != 1) {
							System.err.println("Update failed. "+ update + " code: "+ success);
							this.close();
							return;
						}
					} // end of process datablocks loop
					//if(this.bbr != null)
						//bbr.transmit();
					break;
				case 'q': case 'Q':
					// Ok, we are asked to quit...
					System.out.println("SEND: "+ 'T' + " " +  2  + " " + btsession);
					this.trans.write(intToByteString('T', 1) + intToByteString(2, 3) + intToByteString(btsession, 2));
					this.trans.close();
					isRunning = false;
					isDead = true;
					//if(this.bbr != null)
                        //bbr.close();
					break;
				default:
					// Command byte not recognized...
					// Continue from the beginning, hope for better commands...
					System.out.println("Command not recognized: " + command);
				}
				
			} catch (IOException e) {
				//Could not send...
				this.connectionLost = true;
			} catch (Throwable t){
				System.out.println("Invalid message, discarding.");
				t.printStackTrace();
			}
		}
	}
	
	/**
	 * Returns the bta of the handled BTG.
	 * @return A String containing the bluetooth address of the connected BTG.
	 */
	public String getBluetoothAddress(){
		return this.bta;
	}
	
	/**
	 * Returns the value of the compression algorithm byte. Not used yet.
	 * @return The value of the compression algorithm byte. Not used yet.
	 */
	
	public int getCompression(){
		return this.compression;
	}
	
	public void updateTransceiver(Transceiver trans){
		this.trans = trans;
		connectionLost = false;
		this.notify();
	}
	
	/**
	 * Returns a boolean indicating if the object is alive or not. If the object is not alive,
	 * calling close() will destroy the object. It is impossible to continue reading data from
	 * Betelgeuse after the BetelgeuseProtocolSocket has died, but it is possible to retrieve
	 * information by using the access methods.
	 * @return true, if the object is alive, false otherwise
	 */
	
	public boolean alive(){
		return !isDead;
	}
	
	/**
	 * Closes the connection to the BTG.
	 *
	 */
	
	public synchronized void close(){
		if(isDead){
			//We are waiting to be notified...
			this.notify();
		}else{
			//We are not dead so we are either:
			//A) Continuing to read just like we normally do. This will terminate it.
			isRunning = false;
			//B) Waiting for 5 mins to get a new socket... Wake up -> isRunning is false -> boom
			this.notify();
		}
		this.trans.close();
		if (conn != null){
			try {
				statement.close();
				conn.close();
			} catch (SQLException e) {
				System.err.println("Could not close database connection.");
			}
			conn = null;
		}
		this.isDead = true;
	}
	
	/**
	 * Returns the pooled session id for the BTG handler.
	 * @return The session id used for identifying the BTG handled among current connections.
	 */
	public int getSession(){
		return this.betSession;
	}
	
	/**
	 * Returns the DB session id for the BTG handler.
	 */
	public int getDBSession(){
	    return (int)this.dbSession;
	}
	
	/**
	 * Encodes <emp>number</emp> into a sequence of <emp>size</emp> bytes. 
	 * @param number
	 * @param size
	 * @return a String representation, each character being a byte of the encoding. 
	 */
	private String intToByteString(int number, int size){
		String result = "";
		char c;
		
		for (int i = size-1; i >= 0; i--){
			c = (char)(number % 256);
			result = c + result;
			number = number / 256;
		}
		return result;
	}
	
	public Vector getRules(){
		return this.rules;
	}
	
	public String getUserName(){
		return this.userName;
	}

	public String getParserName() {
		return "Btelgeuse Protocol Socket";
	}
	
	private void intToBytes(int number, int size, ByteArrayOutputStream out){
		byte[] array = new byte[size];
		
		for (int i = size-1; i >= 0; i--){
			array[i] = (byte)(number % 256);
			number = number / 256;
		}
		try{
			out.write(array);
		}catch (IOException e){
			// TODO
		}
	}
}
